import argparse

from scandb.models.db import init_db

from scandb.models.report import ReportVulnByAddressList
from scandb.report.queries import select_vulns, select_ips, select_plugin_ids, select_plugin_by_id
from scandb.report.queries import select_vuln_by_ip, select_vuln_addr_by_plugin, select_vulns_by_plugins
from scandb.report.util import write_to_template, parse_pluginrange

from scandb.statistics.queries import get_vuln_stats
from scandb.statistics.queries import get_port_stats
from scandb.statistics.queries import get_scan_stats
from scandb.statistics.queries import get_host_port_list

def create_list_ReportVulnPlugin(min_severity=0, plugin_ids=[]):
    """
        This function creates a list with ReportVulnPlugin objects.

        :param min_severity: minimum severity (default = 0)
        :type: int
        :return: list of ReportVulnPlugin objects
        :rtype: list
        """
    result = []
    if len(plugin_ids) > 0:
        ids = plugin_ids
    else:
        ids = select_plugin_ids(min_severity=min_severity)
    for id in ids:
        plugin = select_plugin_by_id(id)
        addresses = select_vuln_addr_by_plugin(id)
        plugin.addresses = addresses
        result.append(plugin)
    return result


def create_list_ReportVulnByAddressList(min_severity=0, plugin_ids=[]):
    result = []
    if len(plugin_ids) > 0:
        ips = [v.host.address for v in select_vulns_by_plugins(plugin_ids)]
    else:
        ips = select_ips(min_severity=min_severity)
    for ip in ips:
        addr = ReportVulnByAddressList(address=ip)
        addr.vulns = select_vuln_by_ip(ip=ip, min_severity=min_severity)
        result.append(addr)
    return result


def report_cli():
    parser = argparse.ArgumentParser(description="Generate DOCX reports based on custom templates. "
                                    "See https://bitbucket.org/cbless/scandb/src/master/examples/ for example templates."
                                    "A description of usable objects and their attributes can be found under: \n"
                                    "https://bitbucket.org/cbless/scandb/wiki/Report-Templates")
    parser.add_argument("--db", type=str, required=False, default="scandb.sqlite")
    parser.add_argument("--min-severity", type=int, required=False, default=0,
                        help="Minimum severity level (default: 0). Either plugins or min-severity can be used.")
    parser.add_argument("--plugins", nargs='+', required=False, default=None,
                        help="List of plugins to export. Either plugins or min-severity can be used.")
    parser.add_argument("--export-vulns", required=False, choices=['all', 'unsorted', 'host', 'plugin'],
                        default='plugin', help="Can be used to specifiy how the vulnerabilities will be injected into "
                                               "the template. 'unsorted' means that the vulnerabilites will be "
                                               "available unsorted as 'vulns'. 'host' means that a list of "
                                               "vulnerabilities is avaialable per host. 'plugin' means that the list "
                                               "of affected systems is available per plugin/vulnerability as "
                                               "'vulns_by_plugin'. 'all' means that all three options are available in "
                                               "the template. (default 'plugin')")
    parser.add_argument("--template", type=str, required=False, default="scandb-template.docx",
                        help="Name of the template to render. Examples can be found under: "
                             "https://bitbucket.org/cbless/scandb/src/master/examples/")
    parser.add_argument("--outfile", type=str, required=False, default="scandb-report.docx",
                        help="Name that is used for the generated report.")
    args = parser.parse_args()

    # initialize the database
    database = init_db(args.db)

    vulns = []
    vulns_by_plugin = []
    vulns_by_host = []

    if args.plugins:
        plugin_list = parse_pluginrange(args.plugins)
        if len (plugin_list) > 0:
            if args.export_vulns in ['all', 'vulns']:
                vulns = select_vulns_by_plugins(plugin_list)
            if args.export_vulns in ['all', 'plugin']:
                create_list_ReportVulnPlugin(plugin_ids=plugin_list)
            if args.export_vulns in ['all', 'host']:
                create_list_ReportVulnByAddressList(plugin_ids=plugin_list)
    else:
        # select by min-severity (default = 0)
        if args.export_vulns in ['all', 'vulns']:
            vulns = select_vulns(args.min_severity)
        if args.export_vulns in ['all', 'plugin']:
            vulns_by_plugin = create_list_ReportVulnPlugin(args.min_severity)
        if args.export_vulns in ['all', 'host']:
            vulns_by_host = create_list_ReportVulnByAddressList(args.min_severity)

    scan_stats = get_scan_stats(args.db)
    vuln_stats = get_vuln_stats(args.db)
    port_stats = get_port_stats(args.db)
    host_port_list = get_host_port_list(args.db)

    write_to_template(args.template, outfile=args.outfile, vulns=vulns, vulns_by_plugin=vulns_by_plugin,
                      host_port_list=host_port_list, vulns_by_host=vulns_by_host, vuln_stats=vuln_stats,
                      port_stats=port_stats, scan_stats=scan_stats)


import json


def xref2cve(xref):
    """
    Function to parse the xref field of the nessus plugin to extract a list of CVE numbers.

    :param xref: Json object with vulnerability references
    :type xref: JSON

    :return: string with a list of comma separated CVE numbers
    :rtype: str
    """
    cve = ""
    try:
        tmp = json.loads(xref.replace("'", "\""))

        if 'cve' in tmp:
            cve = ", ".join(tmp['cve'])
    except:
        pass
    return cve


def xref2bid(xref):
    """
    Function to parse the xref field of the nessus plugin to extract a list of BID numbers.

    :param xref: Json object with vulnerability references
    :type xref: JSON

    :return: string with a list of comma separated BID numbers
    :rtype: str
    """
    bid = ""
    try:
        tmp = json.loads(xref.replace("'", "\""))
        if 'bid' in tmp:
            bid = ",".join(tmp['bid'])
    except:
        pass
    return bid


class ReportVuln(object):
    """
    Object with details of an identified vulnerability and information about the vulnerability scanner plugin that was
    used to identify the vulnerability.
    """
    def __init__(self, address="", description="", synopsis="", port="", protocol="", service="", solution="",
                 severity="", xref="", info="", plugin_id="", plugin_name="", plugin="", plugin_family="",
                 plugin_output="",  risk=""):
        """
        Constructor

        :param address: ip address
        :type address: str

        :param description: Vulnerability description
        :type description: str

        :param synopsis:  Nessus synopsis field / Summary
        :type synopsis: str

        :param port: Port number for the associated finding
        :type port: str

        :param protocol: used transport layer protocol (TCP / UDP)
        :type protocol: str

        :param service: detected service (e.g. www, ftp )
        :type service: str

        :param solution: Vulnerability solution / Mitigation measure
        :type solution: str

        :param severity: Severity rating of the vulnerability (4=critical, 3=High, 2=Medium, 1=Low, 0=Informational)
        :type severity: str

        :param xref: unparsed xref ouput from lib-nessus (JSON)
        :type xref: JSON string

        :param info: unparsed info ouput from lib-nessus (JSON)
        :type info: JSON string

        :param plugin_id: Nessus plugin id
        :type plugin_id: str

        :param plugin_name: Nessus plugin name
        :type plugin_name: str

        :param plugin: unparsed plugin ouput from lib-nessus (JSON)
        :type plugin: JSON string

        :param plugin_family: Nessus plugin plugin_family
        :type plugin_family: str

        :param plugin_output: Nessus plugin output
        :type plugin_output: str

        :param risk: unparsed risk from lib-nessus (JSON)
        :type risk: JSON string
        """
        self._address = address
        self._description = description
        self._synopsis = synopsis
        self._port = str(port)
        self._protocol = protocol
        self._service = service
        self._solution = solution
        self._severity = str(severity)
        self._xref = xref
        self._cve = xref2cve(xref)
        self._bid = xref2bid(xref)
        try:
            self._info = json.loads(info)
        except:
            self._info = info
        self._plugin_id = plugin_id
        self._plugin_name = plugin_name
        try:
            self._plugin = json.loads(plugin)
        except:
            self._plugin = plugin
        self._plugin_family = plugin_family
        self._plugin_output = plugin_output
        self._risk = risk

    @property
    def address(self):
        return self._address

    @address.setter
    def address(self, address=""):
        self._address = address

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, description=""):
        self._description = description

    @property
    def synopsis(self):
        return self._synopsis

    @synopsis.setter
    def synopsis(self, synopsis=""):
        self._synopsis = synopsis

    @property
    def port(self):
        return self._port

    @port.setter
    def port(self, port=""):
        self._port = str(port)

    @property
    def protocol(self):
        return self._protocol

    @protocol.setter
    def protocol(self, protocol=""):
        self._protocol = protocol

    @property
    def service(self):
        return self._service

    @service.setter
    def service(self, service=""):
        self._service = service

    @property
    def solution(self):
        return self._solution

    @solution.setter
    def solution(self, solution=""):
        self._solution = solution

    @property
    def severity(self):
        return self._severity

    @severity.setter
    def severity(self, severity="0"):
        self._severity = str(severity)

    @property
    def xref(self):
        return self._xref

    @xref.setter
    def xref(self, xref={}):
        self._xref = xref

    @property
    def cve(self):
        return self._cve

    @cve.setter
    def cve (self, xref=""):
        self._cve = xref2cve(xref)

    @property
    def bid(self):
        return self._bid

    @bid.setter
    def bid(self, value):
        self._bid = xref2bid(value)

    @property
    def info(self):
        return self._info

    @info.setter
    def info(self, value):
        self._info = value

    @property
    def plugin_id(self):
        return self.plugin_id

    @plugin_id.setter
    def plugin_id(self, value):
        self._plugin_id = value

    @property
    def plugin_name(self):
        return self._plugin_name

    @plugin_name.setter
    def plugin_name(self, value):
        self.plugin_name = value

    @property
    def plugin(self):
        return self._plugin
    
    @plugin.setter
    def plugin(self, value):
        self._plugin = value

    @property
    def plugin_family(self):
        return self._plugin_family
    
    @plugin_family.setter
    def plugin_family(self, value):
        self._plugin_family = value
        
    @property
    def plugin_output(self):
        return self._plugin_output
    
    @plugin_output.setter
    def plugin_output(self, value):
        self._plugin_output = value
        
    @property
    def risk(self):
        return self._risk
    
    @risk.setter
    def risk(self, value):
        self._risk = value

    @property
    def exploitability_ease(self):
        if 'exploitability_ease' in self._info:
            return self._info['exploitability_ease']
        return ""

    @property
    def exploit_available(self):
        if 'exploit_available' in self._info:
            return self._info['exploit_available']
        return ""

    @property
    def canvas(self):
        if 'canvas' in self._info:
            return self._info['canvas']
        return ""

    @property
    def metasploit(self):
        if 'metasploit' in self._info:
            return self._info['metasploit']
        return ""

    @property
    def core_impact(self):
        if 'core_impact' in self._info:
            return self._info['core_impact']
        return ""

    @property
    def canvas_name(self):
        if 'canvas_name' in self._info:
            return self._info['canvas_name']
        return ""

    @property
    def metasploit_name(self):
        if 'metasploit_name' in self._info:
            return self._info['metasploit_name']
        return ""


class ReportVulnPlugin(object):
    """
    Subset of fields of the ReportVulns class. Used for the vulns_by_plugin list when generating the report. The
    following fields are missing compared to ReportVulns: plugin_output, address, port, protocol, service. Instead of
    these files a list of ReportVulnAddress objects is used that provide this information.
    """
    def __init__(self, addresses=[], description="", synopsis="", solution="", severity="", xref="", info="",
                 plugin_id="", plugin_name="", plugin="", plugin_family="", risk=""):
        """
        Constructor

        :param addresses: ip addresses
        :type addresses: list of scandb.models.report.ReportVulnAddress obejcts

        :param description: Vulnerability description
        :type description: str

        :param synopsis:  Nessus synopsis field / Summary
        :type synopsis: str

        :param solution: Vulnerability solution / Mitigation measure
        :type solution: str

        :param severity: Severity rating of the vulnerability (4=critical, 3=High, 2=Medium, 1=Low,  0=Informational)
        :type severity: str

        :param xref: unparsed xref ouput from lib-nessus (JSON)
        :type xref: JSON string

        :param info: unparsed info ouput from lib-nessus (JSON)
        :type info: JSON string

        :param plugin_id: Nessus plugin id
        :type plugin_id: str

        :param plugin_name: Nessus plugin name
        :type plugin_name: str

        :param plugin: unparsed plugin ouput from lib-nessus (JSON)
        :type plugin: JSON string

        :param plugin_family: Nessus plugin plugin_family
        :type plugin_family: str

        :param risk: unparsed risk from lib-nessus (JSON)
        :type risk: JSON string
        """
        self._addresses = addresses
        self._description = description
        self._synopsis = synopsis
        self._solution = solution
        self._severity = str(severity)
        self._xref = xref
        self._cve = xref2cve(xref)
        self._bid = xref2bid(xref)
        try:
            self._info = json.loads(info)
        except:
            self._info = info
        self._plugin_id = plugin_id
        self._plugin_name = plugin_name
        try:
            self._plugin = json.loads(plugin)
        except:
            self._plugin = plugin
        self._plugin_family = plugin_family
        self._risk = risk

    @property
    def addresses(self):
        return self._addresses

    @addresses.setter
    def addresses(self, addresses=[]):
        self._addresses = addresses

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, description=""):
        self._description = description

    @property
    def synopsis(self):
        return self._synopsis

    @synopsis.setter
    def synopsis(self, synopsis=""):
        self._synopsis = synopsis

    @property
    def solution(self):
        return self._solution

    @solution.setter
    def solution(self, solution=""):
        self._solution = solution

    @property
    def severity(self):
        return self._severity

    @severity.setter
    def severity(self, severity="0"):
        self._severity = str(severity)

    @property
    def xref(self):
        return self._xref

    @xref.setter
    def xref(self, xref={}):
        self._xref = xref

    @property
    def cve(self):
        return self._cve

    @cve.setter
    def cve(self, xref=""):
        self._cve = xref2cve(xref)

    @property
    def bid(self):
        return self._bid

    @bid.setter
    def bid(self, value):
        self._bid = xref2bid(value)

    @property
    def info(self):
        return self._info

    @info.setter
    def info(self, value):
        self._info = value

    @property
    def plugin_id(self):
        return self.plugin_id

    @plugin_id.setter
    def plugin_id(self, value):
        self._plugin_id = value

    @property
    def plugin_name(self):
        return self._plugin_name

    @plugin_name.setter
    def plugin_name(self, value):
        self.plugin_name = value

    @property
    def plugin(self):
        return self._plugin

    @plugin.setter
    def plugin(self, value):
        self._plugin = value

    @property
    def plugin_family(self):
        return self._plugin_family

    @plugin_family.setter
    def plugin_family(self, value):
        self._plugin_family = value

    @property
    def risk(self):
        return self._risk

    @risk.setter
    def risk(self, value):
        self._risk = value

    @property
    def exploitability_ease(self):
        if 'exploitability_ease' in self._info:
            return self._info['exploitability_ease']
        return ""

    @property
    def exploit_available(self):
        if 'exploit_available' in self._info:
            return self._info['exploit_available']
        return ""

    @property
    def canvas(self):
        if 'canvas' in self._info:
            return self._info['canvas']
        return ""

    @property
    def metasploit(self):
        if 'metasploit' in self._info:
            return self._info['metasploit']
        return ""

    @property
    def core_impact(self):
        if 'core_impact' in self._info:
            return self._info['core_impact']
        return ""

    @property
    def canvas_name(self):
        if 'canvas_name' in self._info:
            return self._info['canvas_name']
        return ""

    @property
    def metasploit_name(self):
        if 'metasploit_name' in self._info:
            return self._info['metasploit_name']
        return ""


class ReportVulnAddress(object):
    """
    Obejct to store information about hosts as well as the Nessus plugin output. This object is used as nested object
    in ReportVulnPlugin.
    """

    def __init__(self, address ="", port="", protocol="", service="", plugin_output=""):
        """
        Constructor

        :param address: ip address
        :type address: str

        :param port: Port number for the associated finding
        :type port: str

        :param protocol: used transport layer protocol (TCP / UDP)
        :type protocol: str

        :param service: detected service (e.g. www, ftp )
        :type service: str

        :param plugin_output: Nessus plugin output
        :type plugin_output: str
        """
        self._address = address
        self._port = port
        self._protocol = protocol
        self._service = service
        self._plugin_output = plugin_output

    @property
    def address(self):
        return self._address

    @address.setter
    def address(self, value):
        self._address = value

    @property
    def port(self):
        return self._port

    @port.setter
    def port(self, value):
        self._port = str(value)
        
    @property
    def protocol(self):
        return self._protocol
    
    @protocol.setter
    def protocol(self, value):
        self._protocol = value

    @property
    def service(self):
        return self._service
    
    @service.setter
    def service(self, value):
        self._service = value
        
    @property
    def plugin_output(self):
        return self._plugin_output
    
    @plugin_output.setter
    def plugin_output(self, value):
        self._plugin_output = value


class ReportVulnByAddressList(object):
    def __init__(self, address="", vulns=[]):
        self._address = address
        self._vulns = vulns

    @property
    def address(self):
        return self._address
    
    @address.setter
    def address(self, value):
        self._address = value
    
    @property
    def vulns(self):
        return self._vulns
    
    @vulns.setter
    def vulns(self, value=[]):
        self._vulns = value


class ReportPort(object):
    def __init__(self, port="", protocol="", service="", banner="", status=""):
        self._port = port
        self._protocol = protocol
        self._service = service
        self._banner = banner
        self._status = status

    @property
    def port(self):
        return self._port

    @port.setter
    def port(self, value):
        self._port = str(value)

    @property
    def protocol(self):
        return self._protocol
    
    @protocol.setter
    def protocol(self, value):
        self._protocol = value
        
    @property
    def service(self):
        return self._service
    
    @service.setter
    def service(self, value):
        self._service = value
    
    @property
    def banner(self):
        return self._banner
    
    @banner.setter
    def banner(self, value):
        self._banner = value
        
    @property
    def status(self):
        return self._status
    
    @status.setter
    def status(self, value):
        self._status = value


class ReportHost(object):
    def __init__(self, address ="", hostname="", os ="", os_gen="", status="", tcp=[], udp=[]):
        self._address = address
        self._hostname = hostname
        self._os = os
        self._os_gen = os_gen
        self._status = status
        self._tcp = tcp
        self._udp = udp

    @property
    def address(self):
        return self._address

    @address.setter
    def address(self, value):
        self._address = value

    @property
    def hostname(self):
        return self._hostname

    @hostname.setter
    def hostname(self, value):
        self._hostname = value

    @property
    def os(self):
        return self._os

    @os.setter
    def os(self, value):
        self._os = value

    @property
    def os_gen(self):
        return self._os_gen

    @os_gen.setter
    def os_gen(self, value):
        self._os_gen = value

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, value):
        self._status = value

    @property
    def tcp(self):
        return self._tcp

    @tcp.setter
    def tcp(self, value=[]):
        self._tcp = value

    @property
    def udp(self):
        return self._udp

    @udp.setter
    def udp(self, value=[]):
        self._udp = value


############## Statistic objects

class ReportVulnStat(object):

    def __init__(self, address ="", critical=0, high=0, medium=0, low=0, info=0):
        self._address = address
        self._critical = critical
        self._high = high
        self._medium = medium
        self._low = low
        self._info = info

    @property
    def address(self):
        return self._address
    
    @address.setter
    def address(self, value):
        self._address = value
    
    @property
    def critical(self):
        return self._critical
    
    @critical.setter
    def critical(self, value):
        self._critical = value
        
    @property
    def high(self):
        return self._high
    
    @high.setter
    def high(self, value):
        self._high = value
        
    @property
    def medium(self):
        return self._medium
    
    @medium.setter
    def medium(self, value):
        self._medium = value
        
    @property
    def low(self):
        return self._low
    
    @low.setter
    def low(self, value):
        self._low = value
        
    @property
    def info(self):
        return self._info
    
    @info.setter
    def info(self, value):
        self._info = value

    @staticmethod
    def get_csv_header(delimiter=";"):
        return delimiter.join(["Address", "Critical", "High", "Medium", "Low", "Info"])

    def as_csv(self, delimiter=";"):
        return delimiter.join([self.address, str(self.critical), str(self.high), str(self.medium), str(self.low),
                               str(self.info)])


class ReportPortStat(object):
    def __init__(self, address ="", tcp="", udp=""):
        self._address = address
        self._tcp = tcp
        self._udp = udp
        
    @property
    def address(self):
        return self._address
    
    @address.setter
    def address(self, value):
        self._address = value
        
    @property
    def tcp(self):
        return self._tcp
    
    @tcp.setter
    def tcp(self, value):
        self._tcp = value

    @property
    def udp(self):
        return self._udp
    
    @udp.setter
    def udp(self, value):
        self._udp = value

    @staticmethod
    def get_csv_header(delimiter=";"):
        return delimiter.join(["Address", "TCP", "UDP"])

    def as_csv(self, delimiter=";"):
        return delimiter.join([self.address, self.tcp, self.udp])


class ReportHostPortStat(object):
    def __init__(self, address ="", ports="", protocol=""):
        self._address = address
        self._ports = ports
        self._protocol = protocol
        
    @property
    def address(self):
        return self._address
    
    @address.setter
    def address(self, value):
        self._address = value

    @property
    def ports(self):
        return self._ports
    
    @ports.setter
    def ports(self, value):
        self._ports = value
        
    @property
    def protocol(self):
        return self._protocol
    
    @protocol.setter
    def protocol(self, value):
        self._protocol = value    
    
    @staticmethod
    def get_csv_header(delimiter=";"):
        return delimiter.join(["Address", "Ports", "Protocol"])

    def as_csv(self, delimiter=";"):
        return delimiter.join([self.address, self.ports, self.protocol])


class ReportScanStat(object):
    def __init__(self, id="", type="", start="", end="", elapsed="", hosts_total="", hosts_up="", hosts_down="", name=""):
        self._id = id
        self._type = type
        self._start = start
        self._end = end
        self._elapsed = elapsed
        self._hosts_total = hosts_total
        self._hosts_up = hosts_up
        self._hosts_down = hosts_down
        self._name = name

    @property
    def id(self):
        return self._id

    @id.setter
    def id(self, value):
        self._id = str(value)

    @property
    def type(self):
        return self._type

    @type.setter
    def type(self, value):
        self._type = value

    @property
    def start(self):
        return self._start

    @start.setter
    def start(self, value):
        self._start = str(value)

    @property
    def end(self):
        return self._end

    @end.setter
    def end(self, value):
        self._end = str(value)

    @property
    def elapsed(self):
        return self._elapsed

    @elapsed.setter
    def elapsed(self, value):
        self._elapsed = str(value)

    @property
    def hosts_total(self):
        return self._hosts_total

    @hosts_total.setter
    def hosts_total(self, value):
        self._hosts_total = str(value)

    @property
    def hosts_up(self):
        return self._hosts_up

    @hosts_up.setter
    def hosts_up(self, value):
        self._hosts_up = str(value)

    @property
    def hosts_down(self):
        return self._hosts_down

    @hosts_down.setter
    def hosts_down(self, value):
        self._hosts_down = str(value)

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @staticmethod
    def get_csv_header(delimiter=";"):
        return delimiter.join(["scan id", "Start", "End", "Elapsed", "Hosts total", "Hosts up", "Hosts down",
                               "Parameters"])

    def as_csv(self, delimiter=";"):
        return delimiter.join([self.id, self.start, self.end, self.elapsed, self.hosts_total, self.hosts_up,
                               self.hosts_down, self.name])





import json

from docxtpl import DocxTemplate, RichText
from termcolor import colored

from scandb.models.report import ReportVuln
from scandb.models.report import ReportVulnAddress
from scandb.models.report import ReportVulnPlugin

def db2ReportVuln(v):
    """

    :param dbvuln: Vuln
    :return:
    """
    vuln = ReportVuln ( address =v.host.address, description =v.description, synopsis=v.synopsis, port=v.port,
                        protocol=v.protocol, service=v.service, solution=v.solution, severity=v.severity,
                        xref=v.xref, info=v.info, plugin_id=v.plugin_id, plugin_name=v.plugin_name, plugin=v.plugin,
                        plugin_family=v.plugin_family, plugin_output=v.plugin_output,  risk=v.risk)
    return vuln


def db2ReportVulnPlugin(v):
    """

    :param dbvuln: Vuln
    :return:
    """
    i = 0
    plugin = ReportVulnPlugin ( description =v.description, synopsis=v.synopsis, solution=v.solution, severity=v.severity,
                            xref=v.xref, info=v.info, plugin_id=v.plugin_id, plugin_name=v.plugin_name, plugin=v.plugin,
                            plugin_family=v.plugin_family, risk=v.risk)
    return plugin


def db2ReportVulnAddress(v):
    address = ReportVulnAddress ( address =v.host.address, port=v.port, protocol=v.protocol, service=v.service,
                                    plugin_output=v.plugin_output)
    return address


def parse_pluginrange(plugins):
    pluginlist = []
    if plugins == "" or plugins is None:
        return pluginlist
    for plugin in plugins.split(','):
        if '-' in plugin:
            pluginrange = plugin.split('-')
            for i in range(int(pluginrange[0]), int(pluginrange[1]) + 1):
                # append all plugins in the pluginrange to the new pluginlist
                pluginlist.append(i)
        else:
            # append single plugin to pluginlist
            pluginlist.append(int(plugin))
    return pluginlist


def write_to_template(template, outfile, scan_stats=[], vuln_stats=[], port_stats=[], host_port_list=[],
                      vulns=[], vulns_by_plugin=[], vulns_by_host=[]):
    try:
        # ensure to handle the plugin_output fields a RichText, so that these field doesn't cause an error when they
        # contain html
        for i in vulns_by_plugin:
            for a in i.addresses:
                a.plugin_output = RichText(a.plugin_output)
        for i in vulns:
            i.plugin_output = RichText(i.plugin_output)
        for i in vulns_by_host:
            for v in i.vulns:
                v.plugin_output = RichText(v.plugin_output)
        # inject the given object into the template engine and render the template
        doc = DocxTemplate(template)
        context = {'vulns' : vulns,
                   'scan_stats' : scan_stats,
                   'vuln_stats': vuln_stats,
                   'port_stats' : port_stats,
                   'vulns_by_plugin': vulns_by_plugin,
                   'vulns_by_host': vulns_by_host,
                   'host_port_list': host_port_list}
        doc.render(context)
        doc.save(outfile)
    except Exception as e:
        print(colored("[-] {0}".format(e.message), "red"))




import argparse

from scandb.services.queries import get_host_list, get_host_list_by_tcp, get_host_list_by_udp


def services_cli():
    parser = argparse.ArgumentParser(description="I can be used to generate target lists (ip address lists) that can be"
                                                 " used as input for other tools based on given filters.")
    parser.add_argument("--db", type=str, required=False, default="scandb.sqlite")
    parser.add_argument("--status", type=str, required=False, default="up",
                        help="Status string stored in database (default: up)")
    parser.add_argument("-t", "--tcp", metavar="PORTS", required=False, type=str, default=None,
                        help="Open TCP ports")
    parser.add_argument("-u", "--udp", metavar="PORTS", required=False, type=str, default=None,
                        help="Open UDP ports")
    parser.add_argument("-o", "--operation", metavar="UNION|INTERSECTION", required=False, type=str, default="UNION",
                        help="Operation to combine the sets of TCP and UDP ports (default: UNION)")
    parser.add_argument("--list", required=False, action='store_true', default=False,
                        help="Generate a target list")
    parser.add_argument("-d", "--list-delimiter", required=False, default="\n",
                        help="Delimiter used to separate hosts in the list output")
    parser.add_argument("--list-file", metavar="FILE", required=False, type=str, default=None,
                        help="Generate a file with the targets instead of printing them to stdout")
    args = parser.parse_args()

    if not args.list and args.list_file is None:
        parser.print_usage()
        return

    ips = []

    if args.tcp is None and args.udp is None:
        ips = get_host_list(args.db, args.status)
    else:
        tcp_ports = []
        udp_ports = []
        if args.tcp is not None:
            tcp_ports = get_host_list_by_tcp(args.db, args.tcp.split(","))
        if args.udp is not None:
            udp_ports = get_host_list_by_udp(args.db, args.udp.split(","))

        if args.operation.upper() == 'INTERSECTION':
            ips = set(tcp_ports).intersection(udp_ports)
        else:
            ips = set(tcp_ports).union(udp_ports)

    if args.list:
        print(args.list_delimiter.join(ips))

    if args.list_file:
        with open(args.list_file, 'w') as f:
            f.write("\n".join(ips))





import sqlite3


def get_host_list(db, status):
    conn = sqlite3.connect(db)
    cur = conn.cursor()
    cur.execute("SELECT distinct address FROM host WHERE status like ? ;", (status,))
    rows = cur.fetchall()
    ips = [ x[0] for x in rows]
    conn.close()
    return ips


def get_host_list_by_udp(db, udp):
    conn = sqlite3.connect(db)
    cur = conn.cursor()
    sql = "SELECT distinct address FROM port where port in ({seq_udp}) and protocol = 'udp' and status = 'open' ".format(
            seq_udp=','.join(['?'] * len(udp)))
    cur.execute(sql, udp)
    rows = cur.fetchall()
    ips = [x[0] for x in rows]
    conn.close()
    return ips


def get_host_list_by_tcp(db, tcp):
    conn = sqlite3.connect(db)
    cur = conn.cursor()
    sql = "SELECT distinct address FROM port where port in ({seq_tcp}) and protocol = 'tcp' and status = 'open'".format(
        seq_tcp=','.join(['?'] * len(tcp)))
    cur.execute(sql, tcp)
    rows = cur.fetchall()
    ips = [x[0] for x in rows]
    conn.close()
    return ips


# temp. unused by cli command
def get_host_list_by_both(db, tcp, udp):
    conn = sqlite3.connect(db)
    cur = conn.cursor()
    sql = "SELECT distinct address FROM port WHERE port in ({seq_tcp}) and protocol = 'tcp' and status = 'open'" \
          "or port in ({seq_udp})  and protocol ='udp' and status = 'open'".format(
            seq_tcp=','.join(['?'] * len(tcp)),
            seq_udp=','.join(['?'] * len(udp)))
    cur.execute(sql, (tcp,udp,))
    rows = cur.fetchall()
    ips = [x[0] for x in rows]
    conn.close()
    return ips




import sqlite3

from scandb.models.db import Vuln, Host
from scandb.report.util import db2ReportVuln

HOSTS_BY_SEVERITY = "SELECT distinct address FROM vuln join host on vuln.host_id = host.id WHERE severity >= ?;"
HOSTS_BY_CVE = "SELECT distinct address FROM vuln join host on vuln.host_id = host.id WHERE xref like ? and severity >= ?;"
HOSTS_BY_VULN_DESC = "SELECT distinct address FROM vuln join host on vuln.host_id = host.id WHERE description like ? and severity >= ?;"
HOSTS_BY_PID = "SELECT distinct address FROM vuln join host on vuln.host_id = host.id WHERE plugin_id = ? and severity >= ?;"
HOSTS_BY_PNAME = "SELECT distinct address FROM vuln join host on vuln.host_id = host.id WHERE plugin_name like ? and severity >= ?;"
HOSTS_BY_POUTPUT = "SELECT distinct address FROM vuln join host on vuln.host_id = host.id WHERE plugin_output like ? and severity >= ?;"
HOSTS_BY_IP = "SELECT distinct address FROM vuln join host on vuln.host_id = host.id WHERE address like ? and severity >= ?;"

HOSTS_DETAILS_BY_SEVERITY = "SELECT distinct address,port,protocol,severity,plugin_id,plugin_name, description, solution, info, xref FROM vuln join host on vuln.host_id = host.id WHERE severity >= ?;"
#HOSTS_DETAILS_BY_SEVERITY = "SELECT distinct address,port,protocol,severity, service, plugin_id, plugin_name, description, synopsis, solution, info, xref, plugin, plugin_family, plugin_output, risk FROM vuln join host on vuln.host_id = host.id WHERE severity >= ?;"
HOSTS_DETAILS_BY_CVE = "SELECT distinct address,port,protocol,severity,plugin_id,plugin_name, description, solution, info, xref FROM vuln join host on vuln.host_id = host.id WHERE xref like ? and severity >= ?;"
HOSTS_DETAILS_BY_VULN_DESC = "SELECT distinct address,port,protocol,severity,plugin_id,plugin_name, description, solution, info, xref FROM vuln join host on vuln.host_id = host.id WHERE description like ? and severity >= ?;"
HOSTS_DETAILS_BY_PID = "SELECT distinct address,port,protocol,severity,plugin_id,plugin_name, description, solution, info, xref FROM vuln join host on vuln.host_id = host.id WHERE plugin_id like ? and severity >= ?;"
HOSTS_DETAILS_BY_PNAME = "SELECT distinct address,port,protocol,severity,plugin_id,plugin_name, description, solution, info, xref FROM vuln join host on vuln.host_id = host.id WHERE plugin_name like ? and severity >= ?;"
HOSTS_DETAILS_BY_POUTPUT = "SELECT distinct address,port,protocol,severity,plugin_id,plugin_name, description, solution, info, xref FROM vuln join host on vuln.host_id = host.id WHERE plugin_output like ? and severity >= ?;"
HOSTS_DETAILS_BY_IP = "SELECT distinct address,port,protocol,severity,plugin_id,plugin_name, description, solution, info, xref FROM vuln join host on vuln.host_id = host.id WHERE address like ? and severity >= ?;"


def get_ips_by_filter(db, query="", search = "", min_severity=0):
    conn = sqlite3.connect(db)
    cur = conn.cursor()
    cur.execute(query, (search,min_severity,))
    rows = cur.fetchall()
    ips = [x[0] for x in rows]
    conn.close()
    return ips


def get_details_by_filter(db, query="", search = "", min_severity=0):
    conn = sqlite3.connect(db)
    cur = conn.cursor()
    cur.execute(query, (search,min_severity,))
    rows = cur.fetchall()
    conn.close()
    return rows


def get_ips_by_severity(db, min_severity=0):
    conn = sqlite3.connect(db)
    cur = conn.cursor()
    cur.execute(HOSTS_BY_SEVERITY, (min_severity,))
    rows = cur.fetchall()
    ips = [x[0] for x in rows]
    conn.close()
    return ips


def get_details_by_severity(db, min_severity=0):
    conn = sqlite3.connect(db)
    cur = conn.cursor()
    cur.execute(HOSTS_DETAILS_BY_SEVERITY, (min_severity,))
    rows = cur.fetchall()
    conn.close()
    return rows




import sqlite3

from scandb.models.report import ReportScanStat
from scandb.models.report import ReportVulnStat
from scandb.models.report import ReportPortStat
from scandb.models.report import ReportHostPortStat


def execute_query(db, query):
    conn = sqlite3.connect(db)
    cur = conn.cursor()
    cur.execute(query)
    rows = cur.fetchall()
    conn.close()
    return rows


def get_scan_stats(db):
    """
    This function creates a list with statistics for each scan that has been imported by scandb-importer.

    :param db: sqlite database created by scandb-importer
    :return: list of scandb.report.models.ReportScanStat objects
    """
    sql = "SELECT id,start,end,elapsed,hosts_total,hosts_up,hosts_down,name FROM scan"
    rows = execute_query(db, sql)
    result = []
    for r in rows:
        id , start, end, elapsed, total, up, down, name = r
        # on Nessus scans hosts up and hosts down are 'None'
        # convert numbers and None to strings to ensure that they don't case exceptions when used in format() or join()
        id = str(id); up = str(up); down = str(down); total = str(total)
        stat = ReportScanStat(id=id, start=start, end=end, elapsed=elapsed, hosts_total=total, hosts_up=up,
                              hosts_down=down, name=name)
        result.append(stat)
    return result


def get_vuln_stats(db):
    """
    This function creates a list with a number of vulnerabilities per host and severity. The static is created across
    all imported Nessus scans but each Nessus Plugin-ID is only counted once per host.

    :param db: sqlite database created by scandb-importer
    :return: list with number of vulnerabilities per host
    """
    sql = "SELECT address,\
    COUNT(CASE WHEN severity = 4 THEN 1 END) as CRITICAL, \
    COUNT(CASE WHEN severity = 3 THEN 1 END) as HIGH,\
    COUNT(CASE WHEN severity = 2 THEN 1 END) as MEDIUM,\
    COUNT(CASE WHEN severity = 1 THEN 1 END) as LOW,\
    COUNT(CASE WHEN severity = 0 THEN 1 END) as INFO\
    from \
    ( select distinct address, plugin, severity   from host h left join vuln v on h.id = v.host_id )\
    GROUP by address\
    order by CRITICAL DESC, HIGH DESC, MEDIUM DESC, LOW DESC, INFO DESC;"
    rows = execute_query(db, sql)
    result = []
    for r in rows:
        address, c, h, m, l, i = (r)
        # convert counter values to strings to ensure that they don't case exceptions when used in format() or join()
        c = str(c); h = str(h); m = str(m); l = str(l); i = str(i)
        stat = ReportVulnStat(address=address, critical=c, high=h, medium=m, low=l, info=i)
        result.append(stat)
    return result


def get_port_stats(db):
    """
    This function creates a list with a number of open TCP- and UDP-Ports per host. The static is created across
    all imported Nmap scans (only ports with status 'open' are counted).

    :param db: sqlite database created by scandb-importer
    :return: list with number of TCP- and UDP-Ports per host
    """
    sql = "select address,\
    COUNT(CASE WHEN protocol = 'tcp' THEN 1 END) as TCP,\
    COUNT(CASE WHEN protocol = 'udp' THEN 1 END) as UDP\
    from (select DISTINCT address, port, protocol from port where status = 'open')\
    GROUP by address"
    rows = execute_query(db, sql)
    result = []
    for r in rows:
        address, tcp, udp = (r)
        # convert counter values to strings to ensure that they don't case exceptions when used in format() or join()
        tcp = str(tcp); udp = str(udp)
        stat = ReportPortStat(address=address, tcp=tcp, udp=udp)
        result.append(stat)
    return result


def get_host_port_list(db):
    sql = """
        select address , group_concat(distinct port), protocol from port where protocol = 'tcp' and status='open' group by address
        union
        select address , group_concat(distinct port), protocol from port where protocol = 'udp' and status='open' group by address;"""
    rows = execute_query(db, sql)
    result = []
    for r in rows:
        address, ports, protocol = r
        stat = ReportHostPortStat(address=address, ports=ports, protocol=protocol)
        result.append(stat)
    return result


from scandb.models.db import Vuln, Host, Scan, Port
from scandb.report.util import db2ReportVulnAddress, db2ReportVulnPlugin, db2ReportVuln


def select_plugin_ids(min_severity = 0):
    """
    Returns a list of plugin ids. Only plugins that match the minimum severity level will be present in the list.

    :param min_severity: minimum severity level
    :type min_severity: int

    :return: list of plugin ids
    :rtype: list
    """
    ids = Vuln.select(Vuln.plugin_id).where(Vuln.severity >= min_severity).distinct()
    result = [i.plugin_id for i in ids]
    return result


def select_plugin_by_id(id=0):
    """
    Returns an instance of a ReportVulnPlugin object.

    :param id: plugin id
    :return: instance of a ReportVulnPlugin object
    :rtype: scandb.report.ReportVulnPlugin
    """
    vuln = Vuln.select().where(Vuln.plugin_id == id).first()
    return db2ReportVulnPlugin(vuln)


def select_vuln_addr_by_plugin(pid):
    """
    Returns a list of ReportVulnAddress objects that are affected by a vulnerability with the given Nessus Plugin-ID.

    :param pid: Nessus Plugin-ID
    :return: list of scandb.models.report.ReportVulnAddress objects
    :rtype: list
    """
    result = []
    ip_port_list = []
    vulns = Vuln.select().where(Vuln.plugin_id == pid)
    for v in vulns:
        ip_port = "{0}:{1}".format(v.host.address, v.port)
        if ip_port not in ip_port_list:
            result.append(v)
            ip_port_list.append(ip_port)
    result = [db2ReportVulnAddress(r) for r in result]
    return result


def select_ips(min_severity = 0):
    """
    Returns a list of ip addresses of systems that are affected by a vulnerability with the given minimum severity level.

    :param min_severity: minimum severity level of the Nessus Plugin
    :type min_severity: int

    :return: list of ip addresses
    :rtype: list
    """
    result = Vuln.select(Host.address).join(Host).where(Vuln.severity >= min_severity).distinct()
    ips = [i.host.address for i in result]
    return ips


def select_vuln_by_ip(ip, min_severity=0):
    """
    Returns a list of vulnerabilities that were identified on a given ip address and that have a minimum severity level.

    :param ip: ip address
    :type ip: str

    :param min_severity: minimum severity level of the Nessus Plugin
    :type min_severity: int

    :return: list of scandb.models.report.ReportVuln objects
    :rtype: list
    """
    result = []
    plugin_port_list = []
    vulns = Vuln.select(Vuln).join(Host).where(Host.address == ip and Vuln.severity >= min_severity )
    for v in vulns:
        plugin_port = "{0}:{1}".format(v.plugin_id, v.port)
        if plugin_port not in plugin_port_list:
            result.append(v)
            plugin_port_list.append(plugin_port)
    return [db2ReportVuln(v) for v in result]



def select_vulns(min_severity=0):
    """
    Returns a list of vulnerabilities with the given minimum severity level.

    :param min_severity: minimum severity level of the Nessus Plugin
    :type min_severity: int

    :return: list of scandb.models.report.ReportVuln objects
    :rtype: list
    """
    result = Vuln.select(Vuln).join(Host).where(Vuln.severity >= min_severity)
    vulns = [db2ReportVuln(v) for v in result]
    return vulns



def select_vulns_by_plugins(ids=[]):
    """
    Returns a list of vulnerabilities that have been identified and where the nessus plugin ID is in the given list of
     plugin IDs.

    :param ids: list of Nessus plugin IDs.
    :type ids: list

    :return: list of scandb.models.report.ReportVuln objects
    :rtype: list
    """
    result = Vuln.select(Vuln).join(Host).where(Vuln.plugin_id.in_(ids)).order_by(Vuln.plugin_id)
    vulns = [db2ReportVuln(v) for v in result]
    return vulns





import argparse
import json

from scandb.models.db import init_db
from scandb.vulns.queries import get_ips_by_filter
from scandb.vulns.queries import get_details_by_filter
from scandb.vulns.queries import get_ips_by_severity
from scandb.vulns.queries import get_details_by_severity

from scandb.vulns.queries import *

def handle_list_ips(args):
    ips = []
    if args.search is not None:
        if args.filter_by == "cve":
            cve = "%{0}%".format(args.search)
            ips = get_ips_by_filter(args.db, HOSTS_BY_CVE, cve, args.min_severity)

        elif args.filter_by == "plugin-id":
            ips = get_ips_by_filter(args.db, HOSTS_BY_PID, args.search, args.min_severity)

        elif args.filter_by == "plugin-name":
            pn = "%{0}%".format(args.search)
            ips = get_ips_by_filter(args.db, HOSTS_BY_PNAME, pn, args.min_severity)

        elif args.filter_by == "description":
            desc = "%{0}%".format(args.search)
            ips = get_ips_by_filter(args.db, HOSTS_BY_VULN_DESC, desc, args.min_severity)

        elif args.filter_by == "plugin-output":
            po = "%{0}%".format(args.search)
            ips = get_ips_by_filter(args.db, HOSTS_BY_POUTPUT, po, args.min_severity)

        elif args.filter_by == "ip":
            ip = "%{0}%".format(args.search)
            ips = get_ips_by_filter(args.db, HOSTS_BY_IP, ip, args.min_severity)
    else:
        ips = get_ips_by_severity(args.db, args.min_severity)
    return ips


def handle_details_list(args):
    details = []

    if args.search is not None:
        if args.filter_by == "cve":
            cve = "%{0}%".format(args.search)
            details = get_details_by_filter(args.db, HOSTS_DETAILS_BY_CVE, cve, args.min_severity)
        elif args.filter_by == "plugin-id":
            details = get_details_by_filter(args.db, HOSTS_DETAILS_BY_PID, args.search, args.min_severity)
        elif args.filter_by == "plugin-name":
            pn = "%{0}%".format(args.search)
            details = get_details_by_filter(args.db, HOSTS_DETAILS_BY_PNAME, pn, args.min_severity)
        elif args.filter_by == "description":
            desc = "%{0}%".format(args.search)
            details = get_details_by_filter(args.db, HOSTS_DETAILS_BY_VULN_DESC, desc, args.min_severity)
        elif args.filter_by == "plugin-output":
            po = "%{0}%".format(args.search)
            details = get_details_by_filter(args.db, HOSTS_DETAILS_BY_POUTPUT, po, args.min_severity)
        elif args.filter_by == "ip":
            ip = "%{0}%".format(args.search)
            details = get_details_by_filter(args.db, HOSTS_DETAILS_BY_IP, ip, args.min_severity)
    else:
        details = get_details_by_severity(args.db, args.min_severity)

    return details


def vulns_cli():
    parser = argparse.ArgumentParser(description="I can be used to query the sqlite database to filter specific "
                                                 "vulnerabilities. Results can be displayed to stdout or written to "
                                                 "a csv file.")
    parser.add_argument("--db", type=str, required=False, default="scandb.sqlite")
    parser.add_argument("--min-severity", type=int, required=False, default=0,
                        help="Minimum severity level (default: 0)")
    parser.add_argument("--filter-by", required=False, choices=['cve', 'plugin-id', 'plugin-name', 'plugin-output',
                                                                'description', 'ip'],
                        default='description', help="Filter hosts by the given filter. The search value is specified "
                                                    "with option --search. The following fields can be used as filter "
                                                    "'cve', 'plugin-id', 'plugin-name', 'description', 'ip'. (Note: "
                                                    "The option 'ip' returns just the ip itself, when '--list ips' "
                                                    "is selected and a vulnerability was detected for that ip, "
                                                    "otherwise the result is empty.) ")
    parser.add_argument("--search", metavar="SEARCH-Term", required=False, type=str, default=None,
                        help="Search term used for querying the database. The type of the search field can be selected "
                             "with the parameter --filter-by")
    parser.add_argument("--list", required=False, choices=['ips', 'details'], default='ips',
                        help="Generate a target list of ip addresses when selecting 'ips' or display the columns "
                             "Address,Port,Protocol,Severity,Plugin-ID,Plugin-Name")
    parser.add_argument("-d", "--list-delimiter", required=False, default="\n",
                        help="Delimiter used to separate hosts in the list output. Only when --list ips is used.")
    parser.add_argument("--list-file", metavar="FILE", required=False, type=str, default=None,
                        help="Generate a file with the results instead of printing them to stdout. Incase of "
                             "'--list ips' is selected the file contains a list of ip address (one per line), "
                             "in case of '--list details' it will be a csv file")
    args = parser.parse_args()


    # initialize the database
    database = init_db(args.db)

    if not args.list and args.list_file is None:
        parser.print_usage()
        return

    ips = []
    details = []

    if args.list == 'ips':
        ips = handle_list_ips(args)
        print(args.list_delimiter.join(ips))

        if args.list_file:
            with open(args.list_file, 'w') as f:
                f.write("\n".join(ips))

    if args.list == 'details':
        details = handle_details_list(args)

        fmt = '{0:>20}{1:>15}{2:>15}{3:>15}{4:>15}{5}'
        print(fmt.format("Address","Port","Protocol","Severity","Plugin-ID","Plugin-Name"))
        for i in details:
            addr, port,proto,severity, pid, pn, description, solution, info, xref = i
            print(fmt.format(addr, port,proto,severity, pid, pn))

        if args.list_file:
            result = ["Address;Port;Protocol;Severity;Plugin-ID;Plugin-Name;CVE"]
            fmt = '{0};{1};{2};{3};{4};{5};{6}'
            for i in details:
                addr, port, proto, severity, pid, pn, description, solution, info, xref = i
                tmp = json.loads(xref.replace("'","\""))
                cve = ""
                if 'cve' in tmp:
                    cve = ",".join(tmp['cve'])
                result.append(fmt.format(addr, port, proto, severity, pid, pn, cve))
            with open (args.list_file , "w") as f:
                f.write("\n".join(result))




import argparse

from scandb.models.report import ReportVulnStat
from scandb.models.report import ReportPortStat
from scandb.models.report import ReportHostPortStat
from scandb.models.report import ReportScanStat

from scandb.report.cli import write_to_template

from scandb.statistics.queries import get_scan_stats
from scandb.statistics.queries import get_port_stats
from scandb.statistics.queries import get_host_port_list
from scandb.statistics.queries import get_vuln_stats


def handle_scan_stats(db, outfile, write_file = False, delimiter= ";", gen_docx=False, template=""):
    results_csv = []
    results_cli = []
    fmt = '{0:^10}{1:^15}{2:^15}{3:^12}{4:^12}{5:^12}{6:^15}{7}'
    results_cli.append(fmt.format("scan id", "Start", "End", "Elapsed", "Hosts total", "Hosts up", "Hosts down",
                                  "Parameters"))
    results_csv.append(ReportScanStat.get_csv_header(delimiter=delimiter))

    stats = get_scan_stats(db)
    for s in stats:
        stat = (s)
        results_cli.append(fmt.format(stat.id, stat.start, stat.end, stat.elapsed, stat.hosts_total, stat.hosts_up,
                                      stat.hosts_down, stat.name))
        results_csv.append(s.as_csv(delimiter=delimiter))

    print("\n".join(results_cli))
    if write_file:
        filename = "{0}-scan-statistics.csv".format(outfile)
        with open(filename, "w") as f:
            f.write("\n".join(results_csv))
        print("Results written to : {0}".format(filename))
    if gen_docx:
        filename="{0}-scan-statistics.docx".format(outfile)
        write_to_template(template=template, outfile=filename, scan_stats=stats)


def handle_vuln_stats(db, outfile, write_file = False, delimiter=";", gen_docx=False, template=""):
    results_csv = []
    results_cli = []
    fmt = '{0:>20}{1:>15}{2:>15}{3:>15}{4:>15}{5:>15}'
    results_cli.append(fmt.format("Address", "Critical", "High", "Medium", "Low", "Info"))
    results_csv.append(ReportVulnStat.get_csv_header(delimiter=delimiter))

    stats = get_vuln_stats(db)

    for s in stats:
        results_cli.append(fmt.format(s.address, s.critical, s.high, s.medium, s.low, s.info))
        results_csv.append(s.as_csv(delimiter=delimiter))

    print("\n".join(results_cli))
    if write_file:
        filename = "{0}-vuln-statistics.csv".format(outfile)
        with open(filename, "w") as f:
            f.write("\n".join(results_csv))
        print("Results written to : {0}".format(filename))
    if gen_docx:
        filename="{0}-vuln-statistics.docx".format(outfile)
        write_to_template(template=template, outfile=filename, vuln_stats=stats)


def handle_port_stats(db, outfile, write_file = False, delimiter=";", gen_docx=False, template=""):
    results_csv = []
    results_cli = []
    fmt = '{0:>20}{1:>15}{2:>15}'
    results_cli.append(fmt.format("Address", "TCP", "UDP"))
    results_csv.append(ReportPortStat.get_csv_header(delimiter=delimiter))

    stats = get_port_stats(db)
    for s in stats:
        results_cli.append(fmt.format(s.address, s.tcp, s.udp))
        results_csv.append(s.as_csv(delimiter=delimiter))

    print("\n".join(results_cli))
    if write_file:
        filename = "{0}-port-statistics.csv".format(outfile)
        with open(filename, "w") as f:
            f.write("\n".join(results_csv))
        print("Results written to : {0}".format(filename))

    if gen_docx:
        filename="{0}-port-statistics.docx".format(outfile)
        write_to_template(template=template, outfile=filename, port_stats=stats)


def handle_host_port_list(db, outfile, delimiter=";", gen_docx=False, template=""):
    rows = get_host_port_list(db)

    results_csv = [ReportHostPortStat.get_csv_header(delimiter=delimiter)]

    for r in rows:
        results_csv.append(r.as_csv(delimiter=delimiter))

    filename = "{0}-hostportlist.csv".format(outfile)
    with open(filename,'w') as f:
        f.write("\n".join(results_csv))
    print ("Results written to : {0}".format(filename))

    if gen_docx:
        filename="{0}-hostportlist.docx".format(outfile)
        write_to_template(template=template, outfile=filename, host_portlist=rows)


def statistics_cli():
    parser = argparse.ArgumentParser(description="I can generate statistics about vulnerabilities, open ports or "
                                                 "for the imported scans. Furthermore I can generate a host/portlist "
                                                 "as csv file. All statistics can be displayed on stdout or they can "
                                                 "be written to csv or docx files (based on templates). "
                                                 "See https://bitbucket.org/cbless/scandb/src/master/examples/ for "
                                                 "example templates.A description of usable objects and their "
                                                 "attributes can be found under: "
                                                 "https://bitbucket.org/cbless/scandb/wiki/Report-Templates")
    parser.add_argument("--db", type=str, required=False, default="scandb.sqlite")
    parser.add_argument("-s", "--scan-statistics", required=False, action='store_true', default=False,
                        help="Print statistics for each scan")
    parser.add_argument("-v", "--vuln-statistics", required=False, action='store_true', default=False,
                        help="Print number of vulns foreach host.")
    parser.add_argument("-p", "--port-statistics", required=False, action='store_true', default=False,
                        help="Print number of 'open' TCP and UDP ports foreach host.")
    parser.add_argument("--host-portlist", required=False, action='store_true', default=False,
                        help="generate a csv with a list of TCP and UDP Ports per host")
    parser.add_argument("-d", "--delimiter", required=False, type=str,  default=";", help="Delimiter for CSV files.")
    parser.add_argument("-o", "--outfile", required=False, default="scandb", help="Prefix for output files.")
    parser.add_argument("-w", "--write-file", required=False, action='store_true', default=False,
                        help="Write data to CSV file. Prefix of filename can be changed with parameter outfile")
    parser.add_argument("--docx", required=False, action='store_true', default=False,
                        help="Render the given DOCX template for the selected statistics. Prefix of filename can be "
                             "changed with parameter '--outfile'. The template can be specified with parameter "
                             "'--template'")
    parser.add_argument("--template", type=str, required=False, default="scandb-template.docx",
                        help="Name of the template to render. Examples can be found under: "
                             "https://bitbucket.org/cbless/scandb/src/master/examples/")

    args = parser.parse_args()

    do_stats = False
    if args.scan_statistics or args.vuln_statistics or args.port_statistics:
        do_stats = True

    if not do_stats and not args.host_portlist:
        parser.print_usage()
        return

    if args.scan_statistics:
        handle_scan_stats(args.db, args.outfile, args.write_file, args.delimiter, args.docx, args.template)

    if args.vuln_statistics:
        handle_vuln_stats(args.db, args.outfile, args.write_file, args.delimiter, args.docx, args.template)

    if args.port_statistics:
        handle_port_stats(args.db, args.outfile, args.write_file, args.delimiter, args.docx, args.template)

    if args.host_portlist:
        handle_host_port_list(args.db, args.outfile, args.docx, args.template)